# dmedia: distributed media library
# Copyright (C) 2012 Novacut Inc
#
# This file is part of `dmedia`.
#
# `dmedia` is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# `dmedia` is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with `dmedia`.  If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#   Jason Gerard DeRose <jderose@novacut.com>

"""
Small helpers for starting threads and processes.
"""

import threading
import multiprocessing
from queue import Queue


def create_thread(target, *args, **kw):
    thread = threading.Thread(target=target, args=args, kwargs=kw)
    thread.daemon = True
    return thread


def start_thread(target, *args, **kw):
    thread = create_thread(target, *args, **kw)
    thread.start()
    return thread


def start_process(target, *args, **kw):
    process = multiprocessing.Process(target=target, args=args, kwargs=kw)
    process.daemon = True
    process.start()
    return process


class SmartQueue(Queue):
    """
    Queue with custom get() that raises exception instances from the queue.
    """

    def get(self, block=True, timeout=None):
        item = super().get(block, timeout)
        if isinstance(item, Exception):
            raise item
        return item

